import { useEffect } from 'react';
import { IOtpCodeView } from '../view/OtpCodeView';

interface IOtpCodeVm {
  otpChar: React.MutableRefObject<HTMLInputElement[]>;
}

type useOtpCodeReturnType = Pick<IOtpCodeView, 'eventHandlers'>;

const useOtpCodeVm = (dependencies: IOtpCodeVm): useOtpCodeReturnType => {
  const { otpChar } = dependencies;

  function focusToInput(target: HTMLInputElement, inputTabIndex: number) {
    const input = target.parentElement?.querySelector(`input[tabindex="${inputTabIndex}"]`) as
      | HTMLInputElement
      | undefined;

    if (!input) return;

    setTimeout(() => input.focus(), 150);
  }

  const handleFocusFirstInput = () => {
    if (!otpChar.current.length) return;

    otpChar.current[0].focus();
  };

  useEffect(() => {
    handleFocusFirstInput();
  }, []);

  const handleFocusInput = (e: React.FocusEvent) => {
    const target = e.target as HTMLInputElement;

    // check previous inputs are not empty
    const currentIndex = target.getAttribute('tabindex');

    if (!currentIndex || +currentIndex === 1) {
      target.select();
      return;
    }

    // get first previous empty
    let isFindEmptyInput = false;
    const firstEmptyInput = otpChar.current.find((item) => {
      const otpItemIndex = item.getAttribute('tabindex');

      if (!otpItemIndex) return false;

      const isInputEmpty = !item.value.trim() || item.value.trim() === '_';

      if (+otpItemIndex < +currentIndex && isInputEmpty && !isFindEmptyInput) {
        isFindEmptyInput = true;
        return true;
      }

      return false;
    });

    if (firstEmptyInput) {
      firstEmptyInput.select();
      return;
    }

    // focus to it
    target.select();
  };

  function goToPreviousInput(target: HTMLInputElement, currentIndex: number) {
    const preIndex = +currentIndex - 1;

    if (!preIndex) return;

    focusToInput(target, preIndex);
  }

  function goToNextInput(target: HTMLInputElement, currentIndex: number) {
    // get next index
    const nextIndex = +currentIndex + 1;

    if (!nextIndex) return;

    focusToInput(target, nextIndex);
  }

  const handleKeyPressInput = (e: React.KeyboardEvent) => {
    // target
    const target = e.target as HTMLInputElement;

    // get current index
    const currentIndex = target.getAttribute('tabindex');

    if (!currentIndex) return;

    const isRemoveChar = e.key.toLowerCase() === 'backspace';

    if (isRemoveChar && +currentIndex !== 1) {
      goToPreviousInput(target, +currentIndex);
      return;
    }

    goToNextInput(target, +currentIndex);
  };

  return {
    eventHandlers: {
      handleFocusInput,
      handleKeyPressInput,
    },
  };
};

export default useOtpCodeVm;
